// This is an object-oriented-style implementation of a timer
// object written in C

#include <stdio.h>
#include <stdlib.h>
#include <inttypes.h>
#include <time.h>

#include "timer.h"

struct timer {
  struct timespec start;
  struct timespec stop;
};

Timer timer_new() {
  return  (Timer)malloc(sizeof(struct timer));
}

void timer_destroy(Timer timer) {
  if (timer) free(timer);
}

void timer_start(Timer timer) {
  if ( !timer ) {
    fprintf(stderr, "timer_start called with NULL\n");
    exit(1);
  }
  clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &(timer->start));
}

void timer_stop(Timer timer) {
  if ( !timer ) {
    fprintf(stderr, "timer_stop called with NULL\n");
    exit(1);
  }
  clock_gettime(CLOCK_PROCESS_CPUTIME_ID, &(timer->stop));
}

double timer_elapsed(Timer timer) {
  double result;
  if ((timer->stop.tv_nsec - timer->start.tv_nsec)<0) {
    result = timer->stop.tv_sec - timer->start.tv_sec - 1;
    result += 1 +(timer->stop.tv_nsec - timer->start.tv_nsec) / 1000000000;
  } else {
    result  = timer->stop.tv_sec - timer->start.tv_sec;
    result += (timer->stop.tv_nsec - timer->start.tv_nsec) / 1000000000.0;
  }
  return result;
}

